@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 //// Prototypes
 ////
 
-extern void generate_kpk_bitbase(uint8 bitbase[]);
+extern void generate_kpk_bitbase(uint8_t bitbase[]);
 
 
 #endif // !defined(BITBASE_H_INCLUDED)
