@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 //// Includes
 ////
 
+#include <cstdlib>
 #include <iostream>
 
 #include "bitboard.h"
@@ -85,10 +86,13 @@ const Bitboard InFrontBB[2][8] = {
   }
 };
 
+#if defined(USE_COMPACT_ROOK_ATTACKS)
 
-#if defined(USE_32BIT_ATTACKS)
+Bitboard RankAttacks[8][64], FileAttacks[8][64];
 
-const uint64 RMult[64] = {
+#elif defined(USE_32BIT_ATTACKS)
+
+const uint64_t RMult[64] = {
   0xd7445cdec88002c0ULL, 0xd0a505c1f2001722ULL, 0xe065d1c896002182ULL,
   0x9a8c41e75a000892ULL, 0x8900b10c89002aa8ULL, 0x9b28d1c1d60005a2ULL,
   0x15d6c88de002d9aULL, 0xb1dbfc802e8016a9ULL, 0x149a1042d9d60029ULL,
@@ -122,7 +126,7 @@ const int RShift[64] = {
 
 #else // if defined(USE_32BIT_ATTACKS)
 
-const uint64 RMult[64] = {
+const uint64_t RMult[64] = {
   0xa8002c000108020ULL, 0x4440200140003000ULL, 0x8080200010011880ULL, 
   0x380180080141000ULL, 0x1a00060008211044ULL, 0x410001000a0c0008ULL, 
   0x9500060004008100ULL, 0x100024284a20700ULL, 0x802140008000ULL, 
@@ -156,15 +160,15 @@ const int RShift[64] = {
 
 #endif // defined(USE_32BIT_ATTACKS)
 
+#if !defined(USE_COMPACT_ROOK_ATTACKS)
 Bitboard RMask[64];
 int RAttackIndex[64];
 Bitboard RAttacks[0x19000];
-uint8 RMobility[0x19000];
-
+#endif
 
 #if defined(USE_32BIT_ATTACKS)
 
-const uint64 BMult[64] = {
+const uint64_t BMult[64] = {
   0x54142844c6a22981ULL, 0x710358a6ea25c19eULL, 0x704f746d63a4a8dcULL,
   0xbfed1a0b80f838c5ULL, 0x90561d5631e62110ULL, 0x2804260376e60944ULL,
   0x84a656409aa76871ULL, 0xf0267f64c28b6197ULL, 0x70764ebb762f0585ULL,
@@ -198,7 +202,7 @@ const int BShift[64] = {
 
 #else // if defined(USE_32BIT_ATTACKS)
 
-const uint64 BMult[64] = {
+const uint64_t BMult[64] = {
   0x440049104032280ULL, 0x1021023c82008040ULL, 0x404040082000048ULL, 
   0x48c4440084048090ULL, 0x2801104026490000ULL, 0x4100880442040800ULL, 
   0x181011002e06040ULL, 0x9101004104200e00ULL, 0x1240848848310401ULL, 
@@ -235,7 +239,6 @@ const int BShift[64] = {
 Bitboard BMask[64];
 int BAttackIndex[64];
 Bitboard BAttacks[0x1480];
-uint8 BMobility[0x1480];
 
 Bitboard SetMaskBB[64];
 Bitboard ClearMaskBB[64];
@@ -251,13 +254,6 @@ Bitboard BishopPseudoAttacks[64];
 Bitboard RookPseudoAttacks[64];
 Bitboard QueenPseudoAttacks[64];
 
-Bitboard KnightAttackKingMask[64];
-Bitboard BishopAttackKingMask1[64];
-Bitboard BishopAttackKingMask2[64];
-Bitboard RookAttackKingMask1[64];
-Bitboard RookAttackKingMask2[64];
-Bitboard QueenAttackKingMask1[64];
-
 
 ////
 //// Local definitions
@@ -270,13 +266,15 @@ namespace {
   void init_between_bitboards();
   Bitboard sliding_attacks(int sq, Bitboard block, int dirs, int deltas[][2],
                            int fmin, int fmax, int rmin, int rmax);
-  Bitboard index_to_bitboard(int index, int bits, Bitboard mask);
-  void init_sliding_attacks(Bitboard attacks[], uint8 mobility[],
+  Bitboard index_to_bitboard(int index, Bitboard mask);
+  void init_sliding_attacks(Bitboard attacks[],
                             int attackIndex[], Bitboard mask[],
                             const int shift[2], const Bitboard mult[],
                             int deltas[][2]);
   void init_pseudo_attacks();
-  void init_attack_king_masks();
+#if defined(USE_COMPACT_ROOK_ATTACKS)
+  void init_file_and_rank_attacks();
+#endif
 };
 
 
@@ -284,6 +282,19 @@ namespace {
 //// Functions
 ////
 
+/// print_bitboard() prints a bitboard in an easily readable format to the
+/// standard output.  This is sometimes useful for debugging.
+
+void print_bitboard(Bitboard b) {
+  for(Rank r = RANK_8; r >= RANK_1; r--) {
+    std::cout << "+---+---+---+---+---+---+---+---+" << std::endl;
+    for(File f = FILE_A; f <= FILE_H; f++)
+      std::cout << "| " << (bit_is_set(b, make_square(f, r))? 'X' : ' ') << ' ';
+    std::cout << "|" << std::endl;
+  }
+  std::cout << "+---+---+---+---+---+---+---+---+" << std::endl;
+}
+
 
 /// init_bitboards() initializes various bitboard arrays.  It is called during
 /// program initialization.
@@ -295,12 +306,15 @@ void init_bitboards() {
   init_ray_bitboards();
   init_attacks();
   init_between_bitboards();
-  init_sliding_attacks(RAttacks, RMobility, RAttackIndex, RMask, RShift,
+#if defined(USE_COMPACT_ROOK_ATTACKS)
+  init_file_and_rank_attacks();
+#else
+  init_sliding_attacks(RAttacks, RAttackIndex, RMask, RShift,
                        RMult, rookDeltas);
-  init_sliding_attacks(BAttacks, BMobility, BAttackIndex, BMask, BShift,
+#endif
+  init_sliding_attacks(BAttacks, BAttackIndex, BMask, BShift,
                        BMult, bishopDeltas);
   init_pseudo_attacks();
-  init_attack_king_masks();
 }
 
 
@@ -318,7 +332,7 @@ static const int BitTable[64] = {
 
 Square first_1(Bitboard b) {
   b ^= (b - 1);
-  uint32 fold = int(b) ^ int(b >> 32);
+  uint32_t fold = int(b) ^ int(b >> 32);
   return Square(BitTable[(fold * 0x783a9b23) >> 26]);
 }
 
@@ -326,9 +340,9 @@ Square first_1(Bitboard b) {
 /// pop_1st_bit() finds and clears the least significant nonzero bit in a
 /// nonzero bitboard.
 
-Square pop_1st_bit(Bitboard* b) {
+Square pop_1st_bit(Bitboard *b) {
   Bitboard bb = *b ^ (*b - 1);
-  uint32 fold = int(bb) ^ int(bb >> 32);
+  uint32_t fold = int(bb) ^ int(bb >> 32);
   *b &= (*b - 1);
   return Square(BitTable[(fold * 0x783a9b23) >> 26]);
 }
@@ -353,7 +367,7 @@ Square first_1(Bitboard b) {
 /// pop_1st_bit() finds and clears the least significant nonzero bit in a
 /// nonzero bitboard.
 
-Square pop_1st_bit(Bitboard* b) {
+Square pop_1st_bit(Bitboard *b) {
   Bitboard bb = *b;
   *b &= (*b - 1);
   return Square(BitTable[((bb & -bb) * 0x218a392cd3d5dbfULL) >> 58]); 
@@ -451,8 +465,8 @@ namespace {
   }
 
 
-  Bitboard index_to_bitboard(int index, int bits, Bitboard mask) {
-    int i, j;
+  Bitboard index_to_bitboard(int index, Bitboard mask) {
+    int i, j, bits = count_1s(mask);
     Bitboard result = 0ULL;
     for(i = 0; i < bits; i++) {
       j = pop_1st_bit(&mask);
@@ -462,7 +476,7 @@ namespace {
   }
 
 
-  void init_sliding_attacks(Bitboard attacks[], uint8 mobility[],
+  void init_sliding_attacks(Bitboard attacks[],
                             int attackIndex[], Bitboard mask[],
                             const int shift[2], const Bitboard mult[],
                             int deltas[][2]) {
@@ -474,23 +488,16 @@ namespace {
       j = (1 << (64 - shift[i]));
       for(k = 0; k < j; k++) {
 #if defined(USE_32BIT_ATTACKS)
-        b = index_to_bitboard(k, 32 - shift[i], mask[i]);
+        b = index_to_bitboard(k, mask[i]);
         attacks[index + 
                  (unsigned(int(b) * int(mult[i]) ^ 
                            int(b >> 32) * int(mult[i] >> 32)) 
                   >> shift[i])] =
           sliding_attacks(i, b, 4, deltas);
-        mobility[index + 
-                  (unsigned(int(b) * int(mult[i]) ^ 
-                            int(b >> 32) * int(mult[i] >> 32)) 
-                   >> shift[i])] =
-          count_1s(sliding_attacks(i, b, 4, deltas));
 #else
-        b = index_to_bitboard(k, 64 - shift[i], mask[i]);
+        b = index_to_bitboard(k, mask[i]);
         attacks[index + ((b * mult[i]) >> shift[i])] =
           sliding_attacks(i, b, 4, deltas);
-        mobility[index + ((b * mult[i]) >> shift[i])] =
-          count_1s(sliding_attacks(i, b, 4, deltas));
 #endif
       }
       index += j;
@@ -506,31 +513,30 @@ namespace {
       QueenPseudoAttacks[s] = queen_attacks_bb(s, EmptyBoardBB);
     }
   }
-  
 
-  void init_attack_king_masks() {
-    Square s1, s2;
-    Bitboard b;
+#if defined(USE_COMPACT_ROOK_ATTACKS)
+  void init_file_and_rank_attacks() {
+    int i, j, k, l, m, s;
+    Bitboard b1, b2;
+    for(i = 0; i < 64; i++) {
 
-    for(s1 = SQ_A1; s1 <= SQ_H8; s1++) {
-      KnightAttackKingMask[s1] = EmptyBoardBB;
-      BishopAttackKingMask1[s1] = EmptyBoardBB;
-      BishopAttackKingMask2[s1] = EmptyBoardBB;
-      RookAttackKingMask1[s1] = EmptyBoardBB;
-      RookAttackKingMask2[s1] = EmptyBoardBB;
-      QueenAttackKingMask1[s1] = EmptyBoardBB;
-
-      b = StepAttackBB[KING][s1];
-      while(b) {
-        s2 = pop_1st_bit(&b);
-        KnightAttackKingMask[s1] |= StepAttackBB[KNIGHT][s2];
-        BishopAttackKingMask1[s1] |= StepAttackBB[BISHOP][s2];
-        RookAttackKingMask1[s1] |= StepAttackBB[ROOK][s2];
-        QueenAttackKingMask1[s1] |= StepAttackBB[QUEEN][s2];
-        BishopAttackKingMask2[s1] |= bishop_attacks_bb(s2, EmptyBoardBB);
-        RookAttackKingMask2[s1] |= rook_attacks_bb(s2, EmptyBoardBB);
+      for(m = 0; m <= 1; m++) {
+        b1 = 0ULL;
+        for(j = 0; j < 6; j++) if(i & (1<<j)) b1 |= (1ULL << ((j+1)*(1+m*7)));
+        for(j = 0; j < 8; j++) {
+          b2 = 0ULL;
+          for(k = 0, s = 1; k < 2; k++, s *= -1) {
+            for(l = j+s; l >= 0 && l <= 7; l += s) {
+              b2 |= (m? RankBB[l] : FileBB[l]);
+              if(b1 & (1ULL << (l*(1+m*7)))) break;
+            }
+          }
+          if(m) FileAttacks[j][(b1*0xd6e8802041d0c441ULL) >> 58] = b2;
+          else RankAttacks[j][i] = b2;
+        }
       }
     }
-  }     
+  }
+#endif // defined(USE_COMPACT_ROOK_ATTACKS)
     
 }
