@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ namespace {
 
   /// Random numbers from PolyGlot, used to compute book hash keys.
 
-  const uint64 Random64[781] = {
+  const uint64_t Random64[781] = {
     0x9D39247E33776D41ULL, 0x2AF7398005AAA5C7ULL, 0x44DB015024623547ULL,
     0x9C15F73E62A76AE2ULL, 0x75834465489C0C89ULL, 0x3290AC3A203001BFULL,
     0x0FBBAD1F61042279ULL, 0xE83A908FF2FB60CAULL, 0x0D7E765D58755C10ULL,
@@ -333,13 +333,13 @@ namespace {
 
   /// Prototypes
   
-  uint64 book_key(const Position& pos);
-  uint64 book_piece_key(Piece p, Square s);
-  uint64 book_castle_key(const Position& pos);
-  uint64 book_ep_key(const Position& pos);
-  uint64 book_color_key(const Position& pos);
+  uint64_t book_key(const Position &pos);
+  uint64_t book_piece_key(Piece p, Square s);
+  uint64_t book_castle_key(const Position &pos);
+  uint64_t book_ep_key(const Position &pos);
+  uint64_t book_color_key(const Position &pos);
 
-  uint64 read_integer(FILE* file, int size);
+  uint64_t read_integer(FILE *file, int size);
 
 }
 
@@ -359,7 +359,7 @@ Book::Book() {
 
 /// Book::open() opens a book file with a given file name.
 
-void Book::open(const std::string& fName) {
+void Book::open(const std::string &fName) {
   fileName = fName;
   bookFile = fopen(fileName.c_str(), "rb");
   if(bookFile != NULL) {
@@ -404,10 +404,10 @@ const std::string Book::file_name() const {
 /// Book::get_move() gets a book move for a given position.  Returns
 /// MOVE_NONE if no book move is found.
 
-Move Book::get_move(const Position& pos) const {
+Move Book::get_move(const Position &pos) const {
   if(this->is_open()) {
     int bestMove = 0, bestScore = 0, move, score;
-    uint64 key = book_key(pos);
+    uint64_t key = book_key(pos);
     BookEntry entry;
 
     for(int i = this->find_key(key); i < bookSize; i++) {
@@ -441,7 +441,7 @@ Move Book::get_move(const Position& pos) const {
 /// entry with the same key as the input is returned.  When the key is not
 /// found in the book file, bookSize is returned.
 
-int Book::find_key(uint64 key) const {
+int Book::find_key(uint64_t key) const {
   int left, right, mid;
   BookEntry entry;
 
@@ -498,8 +498,8 @@ void Book::read_entry(BookEntry& entry, int n) const {
 
 namespace {
 
-  uint64 book_key(const Position& pos) {
-    uint64 result = 0ULL;
+  uint64_t book_key(const Position &pos) {
+    uint64_t result = 0ULL;
 
     for(Color c = WHITE; c <= BLACK; c++) {
       Bitboard b = pos.pieces_of_color(c);
@@ -523,13 +523,13 @@ namespace {
   }
         
 
-  uint64 book_piece_key(Piece p, Square s) {
+  uint64_t book_piece_key(Piece p, Square s) {
     return Random64[RandomPiece + (PieceTo12[int(p)]^1)*64 + int(s)];
   }
 
 
-  uint64 book_castle_key(const Position& pos) {
-    uint64 result = 0ULL;
+  uint64_t book_castle_key(const Position &pos) {
+    uint64_t result = 0ULL;
 
     if(pos.can_castle_kingside(WHITE))
       result ^= Random64[RandomCastle+0];
@@ -543,19 +543,19 @@ namespace {
   }
 
   
-  uint64 book_ep_key(const Position& pos) {
+  uint64_t book_ep_key(const Position &pos) {
     return (pos.ep_square() == SQ_NONE)?
       0ULL : Random64[RandomEnPassant + square_file(pos.ep_square())];
   }
 
   
-  uint64 book_color_key(const Position& pos) {
+  uint64_t book_color_key(const Position &pos) {
     return (pos.side_to_move() == WHITE)? Random64[RandomTurn] : 0ULL;
   }
   
 
-  uint64 read_integer(FILE* file, int size) {
-    uint64 n = 0ULL;;
+  uint64_t read_integer(FILE *file, int size) {
+    uint64_t n = 0ULL;;
     int i;
     int b;
 
