@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <cassert>
 
 #include "bitbase.h"
-#include "distance.h"
 #include "endgame.h"
 
 
@@ -62,6 +61,13 @@ KRKNEvaluationFunction EvaluateKNKR = KRKNEvaluationFunction(BLACK);
 KQKREvaluationFunction EvaluateKQKR = KQKREvaluationFunction(WHITE);
 KQKREvaluationFunction EvaluateKRKQ = KQKREvaluationFunction(BLACK);
 
+// KBB vs KN:
+KBBKNEvaluationFunction EvaluateKBBKN = KBBKNEvaluationFunction(WHITE);
+KBBKNEvaluationFunction EvaluateKNKBB = KBBKNEvaluationFunction(BLACK);
+
+// K and two minors vs K and one or two minors:
+KmmKmEvaluationFunction EvaluateKmmKm = KmmKmEvaluationFunction(WHITE);
+
 
 /// Scaling functions
 
@@ -97,6 +103,10 @@ KBPKNScalingFunction ScaleKNKBP = KBPKNScalingFunction(BLACK);
 KNPKScalingFunction ScaleKNPK = KNPKScalingFunction(WHITE);
 KNPKScalingFunction ScaleKKNP = KNPKScalingFunction(BLACK);
 
+// KPKP
+KPKPScalingFunction ScaleKPKPw = KPKPScalingFunction(WHITE);
+KPKPScalingFunction ScaleKPKPb = KPKPScalingFunction(BLACK);
+
 
 ////
 //// Local definitions
@@ -106,7 +116,7 @@ namespace {
 
   // Table used to drive the defending king towards the edge of the board
   // in KX vs K and KQ vs KR endgames:
-  const uint8 MateTable[64] = {
+  const uint8_t MateTable[64] = {
     100, 90, 80, 70, 70, 80, 90, 100,
     90, 70, 60, 50, 50, 60, 70, 90,
     80, 60, 40, 30, 30, 40, 60, 80,
@@ -119,7 +129,7 @@ namespace {
 
   // Table used to drive the defending king towards a corner square of the
   // right color in KBN vs K endgames:
-  const uint8 KBNKMateTable[64] = {
+  const uint8_t KBNKMateTable[64] = {
     200, 190, 180, 170, 160, 150, 140, 130,
     190, 180, 170, 160, 150, 140, 130, 140,
     180, 170, 155, 140, 140, 125, 140, 150,
@@ -135,7 +145,7 @@ namespace {
   const int DistanceBonus[8] = {0, 0, 100, 80, 60, 40, 20, 10};
 
   // Bitbase for KP vs K:
-  uint8 KPKBitbase[24576];
+  uint8_t KPKBitbase[24576];
 
   // Penalty for big distance between king and knight for the defending king
   // and knight in KR vs KN endgames:
@@ -151,15 +161,14 @@ namespace {
     return Value(KBNKMateTable[s]);
   }
 
-  inline Value distance_bonus(Distance d) {
+  inline Value distance_bonus(int d) {
     return Value(DistanceBonus[d]);
   }
 
-  inline Value krkn_king_knight_distance_penalty(Distance d) {
+  inline Value krkn_king_knight_distance_penalty(int d) {
     return Value(KRKNKingKnightDistancePenalty[d]);
   }
 
-
   // Function for probing the KP vs K bitbase:
   int probe_kpk(Square wksq, Square wpsq, Square bksq, Color stm);
 
@@ -184,6 +193,8 @@ KRKPEvaluationFunction::KRKPEvaluationFunction(Color c) : EndgameEvaluationFunct
 KRKBEvaluationFunction::KRKBEvaluationFunction(Color c) : EndgameEvaluationFunction(c) { }
 KRKNEvaluationFunction::KRKNEvaluationFunction(Color c) : EndgameEvaluationFunction(c) { }
 KQKREvaluationFunction::KQKREvaluationFunction(Color c) : EndgameEvaluationFunction(c) { }
+KBBKNEvaluationFunction::KBBKNEvaluationFunction(Color c) : EndgameEvaluationFunction(c) { }
+KmmKmEvaluationFunction::KmmKmEvaluationFunction(Color c) : EndgameEvaluationFunction(c) { }
 
 
 ScalingFunction::ScalingFunction(Color c) {
@@ -199,6 +210,7 @@ KPsKScalingFunction::KPsKScalingFunction(Color c) : ScalingFunction(c) { }
 KBPKBScalingFunction::KBPKBScalingFunction(Color c) : ScalingFunction(c) { }
 KBPKNScalingFunction::KBPKNScalingFunction(Color c) : ScalingFunction(c) { }
 KNPKScalingFunction::KNPKScalingFunction(Color c) : ScalingFunction(c) { }
+KPKPScalingFunction::KPKPScalingFunction(Color c) : ScalingFunction(c) { }
 
 
 /// Mate with KX vs K.  This function is used to evaluate positions with
@@ -206,7 +218,7 @@ KNPKScalingFunction::KNPKScalingFunction(Color c) : ScalingFunction(c) { }
 /// attacking side a bonus for driving the defending king towards the edge
 /// of the board, and for keeping the distance between the two kings small.
 
-Value KXKEvaluationFunction::apply(const Position& pos) {
+Value KXKEvaluationFunction::apply(const Position &pos) {
 
   assert(pos.non_pawn_material(weakerSide) == Value(0));
   assert(pos.pawn_count(weakerSide) == Value(0));
@@ -232,7 +244,7 @@ Value KXKEvaluationFunction::apply(const Position& pos) {
 /// Mate with KBN vs K.  This is similar to KX vs K, but we have to drive the
 /// defending king towards a corner square of the right color.
                   
-Value KBNKEvaluationFunction::apply(const Position& pos) {
+Value KBNKEvaluationFunction::apply(const Position &pos) {
 
   assert(pos.non_pawn_material(weakerSide) == Value(0));
   assert(pos.pawn_count(weakerSide) == Value(0));
@@ -261,7 +273,7 @@ Value KBNKEvaluationFunction::apply(const Position& pos) {
 
 /// KP vs K.  This endgame is evaluated with the help of a bitbase.
 
-Value KPKEvaluationFunction::apply(const Position& pos) {
+Value KPKEvaluationFunction::apply(const Position &pos) {
 
   assert(pos.non_pawn_material(strongerSide) == Value(0));
   assert(pos.non_pawn_material(weakerSide) == Value(0));
@@ -305,7 +317,7 @@ Value KPKEvaluationFunction::apply(const Position& pos) {
 /// far advanced with support of the king, while the attacking king is far
 /// away.
 
-Value KRKPEvaluationFunction::apply(const Position& pos) {
+Value KRKPEvaluationFunction::apply(const Position &pos) {
 
   assert(pos.non_pawn_material(strongerSide) == RookValueMidgame);
   assert(pos.pawn_count(strongerSide) == 0);
@@ -360,7 +372,7 @@ Value KRKPEvaluationFunction::apply(const Position& pos) {
 /// KR vs KB.  This is very simple, and always returns drawish scores.  The
 /// score is slightly bigger when the defending king is close to the edge.
 
-Value KRKBEvaluationFunction::apply(const Position& pos) {
+Value KRKBEvaluationFunction::apply(const Position &pos) {
 
   assert(pos.non_pawn_material(strongerSide) == RookValueMidgame);
   assert(pos.pawn_count(strongerSide) == 0);
@@ -376,7 +388,7 @@ Value KRKBEvaluationFunction::apply(const Position& pos) {
 /// KR vs KN.  The attacking side has slightly better winning chances than
 /// in KR vs KB, particularly if the king and the knight are far apart.
 
-Value KRKNEvaluationFunction::apply(const Position& pos) {
+Value KRKNEvaluationFunction::apply(const Position &pos) {
 
   assert(pos.non_pawn_material(strongerSide) == RookValueMidgame);
   assert(pos.pawn_count(strongerSide) == 0);
@@ -400,7 +412,7 @@ Value KRKNEvaluationFunction::apply(const Position& pos) {
 /// for the defending side in the search, this is usually sufficient to be
 /// able to win KQ vs KR.
 
-Value KQKREvaluationFunction::apply(const Position& pos) {
+Value KQKREvaluationFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == QueenValueMidgame);
   assert(pos.pawn_count(strongerSide) == 0);
   assert(pos.non_pawn_material(weakerSide) == RookValueMidgame);
@@ -416,13 +428,43 @@ Value KQKREvaluationFunction::apply(const Position& pos) {
 }
 
 
+Value KBBKNEvaluationFunction::apply(const Position &pos) {
+  assert(pos.bishop_count(strongerSide) == 2);
+  assert(pos.non_pawn_material(strongerSide) == 2*BishopValueMidgame);
+  assert(pos.knight_count(weakerSide) == 1);
+  assert(pos.non_pawn_material(weakerSide) == KnightValueMidgame);
+  assert(pos.pawns() == EmptyBoardBB);
+
+  Value result = BishopValueEndgame;
+  Square wksq = pos.king_square(strongerSide);
+  Square bksq = pos.king_square(weakerSide);
+  Square nsq = pos.knight_list(weakerSide, 0);
+
+  // Bonus for attacking king close to defending king:
+  result += distance_bonus(square_distance(wksq, bksq));
+
+  // Bonus for driving the defending king and knight apart:
+  result += Value(square_distance(bksq, nsq) * 32);
+
+  // Bonus for restricting the knight's mobility:
+  result += Value((8 - count_1s_max_15(pos.knight_attacks(nsq))) * 8);
+
+  return (strongerSide == pos.side_to_move())? result : -result;
+}
+
+
+Value KmmKmEvaluationFunction::apply(const Position &pos) {
+  return Value(0);
+}
+
+
 /// KBPKScalingFunction scales endgames where the stronger side has king,
 /// bishop and one or more pawns.  It checks for draws with rook pawns and a
 /// bishop of the wrong color.  If such a draw is detected, ScaleFactor(0) is
 /// returned.  If not, the return value is SCALE_FACTOR_NONE, i.e. no scaling
 /// will be used.
 
-ScaleFactor KBPKScalingFunction::apply(const Position& pos) {
+ScaleFactor KBPKScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == BishopValueMidgame);
   assert(pos.bishop_count(strongerSide) == 1);
   assert(pos.pawn_count(strongerSide) >= 1);
@@ -430,7 +472,7 @@ ScaleFactor KBPKScalingFunction::apply(const Position& pos) {
   // No assertions about the material of weakerSide, because we want draws to
   // be detected even when the weaker side has some pawns.
 
-  Bitboard pawns = pos.pawns_of_color(strongerSide);
+  Bitboard pawns = pos.pawns(strongerSide);
   File pawnFile = square_file(pos.pawn_list(strongerSide, 0));
 
   if((pawnFile == FILE_A || pawnFile == FILE_H) &&
@@ -474,7 +516,7 @@ ScaleFactor KBPKScalingFunction::apply(const Position& pos) {
 /// It tests for fortress draws with a rook on the third rank defended by
 /// a pawn.
 
-ScaleFactor KQKRPScalingFunction::apply(const Position& pos) {
+ScaleFactor KQKRPScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == QueenValueMidgame);
   assert(pos.queen_count(strongerSide) == 1);
   assert(pos.pawn_count(strongerSide) == 0);
@@ -484,11 +526,11 @@ ScaleFactor KQKRPScalingFunction::apply(const Position& pos) {
   Square kingSq = pos.king_square(weakerSide);
   if(pawn_rank(weakerSide, kingSq) <= RANK_2 &&
      pawn_rank(weakerSide, pos.king_square(strongerSide)) >= RANK_4 &&
-     (pos.rooks_of_color(weakerSide) & relative_rank_bb(weakerSide, RANK_3)) &&
-     (pos.pawns_of_color(weakerSide) & relative_rank_bb(weakerSide, RANK_2)) &&
-     (pos.king_attacks(kingSq) & pos.pawns_of_color(weakerSide))) {
+     (pos.rooks(weakerSide) & relative_rank_bb(weakerSide, RANK_3)) &&
+     (pos.pawns(weakerSide) & relative_rank_bb(weakerSide, RANK_2)) &&
+     (pos.king_attacks(kingSq) & pos.pawns(weakerSide))) {
     Square rsq = pos.rook_list(weakerSide, 0);
-    if(pos.pawn_attacks(strongerSide, rsq) & pos.pawns_of_color(weakerSide))
+    if(pos.pawn_attacks(strongerSide, rsq) & pos.pawns(weakerSide))
       return ScaleFactor(0);
   }
   return SCALE_FACTOR_NONE;
@@ -503,7 +545,7 @@ ScaleFactor KQKRPScalingFunction::apply(const Position& pos) {
 /// It would also be nice to rewrite the actual code for this function,
 /// which is mostly copied from Glaurung 1.x, and not very pretty.
 
-ScaleFactor KRPKRScalingFunction::apply(const Position& pos) {
+ScaleFactor KRPKRScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == RookValueMidgame);
   assert(pos.pawn_count(strongerSide) == 1);
   assert(pos.non_pawn_material(weakerSide) == RookValueMidgame);
@@ -600,6 +642,16 @@ ScaleFactor KRPKRScalingFunction::apply(const Position& pos) {
                   - (8 * square_distance(wpsq, queeningSq) +
                      2 * square_distance(wksq, queeningSq)));
 
+  // If the pawn is not far advanced, and the defending king is somewhere in
+  // the pawn's path, it's probably a draw:
+  if(r <= RANK_4 && bksq > wpsq) {
+    if(square_file(bksq) == square_file(wpsq))
+      return ScaleFactor(10);
+    if(abs(square_file(bksq) - square_file(wpsq)) == 1
+       && square_distance(wksq, bksq) > 2)
+      return ScaleFactor(24 - 2 * square_distance(wksq, bksq));
+  }
+
   return SCALE_FACTOR_NONE;
 }
 
@@ -608,7 +660,7 @@ ScaleFactor KRPKRScalingFunction::apply(const Position& pos) {
 /// single pattern:  If the stronger side has no pawns and the defending king
 /// is actively placed, the position is drawish.
 
-ScaleFactor KRPPKRPScalingFunction::apply(const Position& pos) {
+ScaleFactor KRPPKRPScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == RookValueMidgame);
   assert(pos.pawn_count(strongerSide) == 2);
   assert(pos.non_pawn_material(weakerSide) == RookValueMidgame);
@@ -625,8 +677,7 @@ ScaleFactor KRPPKRPScalingFunction::apply(const Position& pos) {
 
   Rank r = Max(pawn_rank(strongerSide, wpsq1), pawn_rank(strongerSide, wpsq2));
 
-  if(file_distance(bksq, wpsq1) <= Distance(1)
-     && file_distance(bksq, wpsq2) <= Distance(1)
+  if(file_distance(bksq, wpsq1) <= 1 && file_distance(bksq, wpsq2) <= 1
      && pawn_rank(strongerSide, bksq) > r) {
     switch(r) {
 
@@ -647,20 +698,19 @@ ScaleFactor KRPPKRPScalingFunction::apply(const Position& pos) {
 /// against king.  There is just a single rule here:  If all pawns are on
 /// the same rook file and are blocked by the defending king, it's a draw.
 
-ScaleFactor KPsKScalingFunction::apply(const Position& pos) {
+ScaleFactor KPsKScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == Value(0));
   assert(pos.pawn_count(strongerSide) >= 2);
   assert(pos.non_pawn_material(weakerSide) == Value(0));
   assert(pos.pawn_count(weakerSide) == 0);
 
-  Bitboard pawns = pos.pawns_of_color(strongerSide);
+  Bitboard pawns = pos.pawns(strongerSide);
 
   // Are all pawns on the 'a' file?
   if((pawns & ~FileABB) == EmptyBoardBB) {
     // Does the defending king block the pawns?
     Square ksq = pos.king_square(weakerSide);
-    if(square_distance(ksq, relative_square(strongerSide, SQ_A8))
-       <= Distance(1))
+    if(square_distance(ksq, relative_square(strongerSide, SQ_A8)) <= 1)
       return ScaleFactor(0);
     else if(square_file(ksq) == FILE_A &&
        (in_front_bb(strongerSide, ksq) & pawns) == EmptyBoardBB)
@@ -672,8 +722,7 @@ ScaleFactor KPsKScalingFunction::apply(const Position& pos) {
   else if((pawns & ~FileHBB) == EmptyBoardBB) {
     // Does the defending king block the pawns?
     Square ksq = pos.king_square(weakerSide);
-    if(square_distance(ksq, relative_square(strongerSide, SQ_H8))
-       <= Distance(1))
+    if(square_distance(ksq, relative_square(strongerSide, SQ_H8)) <= 1)
       return ScaleFactor(0);
     else if(square_file(ksq) == FILE_H &&
        (in_front_bb(strongerSide, ksq) & pawns) == EmptyBoardBB)
@@ -692,7 +741,7 @@ ScaleFactor KPsKScalingFunction::apply(const Position& pos) {
 /// it's a draw.  If the two bishops have opposite color, it's almost always
 /// a draw.
 
-ScaleFactor KBPKBScalingFunction::apply(const Position& pos) {
+ScaleFactor KBPKBScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == BishopValueMidgame);
   assert(pos.bishop_count(strongerSide) == 1);
   assert(pos.pawn_count(strongerSide) == 1);
@@ -730,7 +779,7 @@ ScaleFactor KBPKBScalingFunction::apply(const Position& pos) {
     else {
       Bitboard ray =
         ray_bb(pawnSq, (strongerSide == WHITE)? SIGNED_DIR_N : SIGNED_DIR_S);
-      if(ray & pos.kings_of_color(weakerSide))
+      if(ray & pos.kings(weakerSide))
         return ScaleFactor(0);
       if((pos.bishop_attacks(weakerBishopSq) & ray)
          && square_distance(weakerBishopSq, pawnSq) >= 3)
@@ -746,7 +795,7 @@ ScaleFactor KBPKBScalingFunction::apply(const Position& pos) {
 /// square of the king is not of the same color as the stronger side's bishop,
 /// it's a draw.
 
-ScaleFactor KBPKNScalingFunction::apply(const Position& pos) {
+ScaleFactor KBPKNScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == BishopValueMidgame);
   assert(pos.bishop_count(strongerSide) == 1);
   assert(pos.pawn_count(strongerSide) == 1);
@@ -772,7 +821,7 @@ ScaleFactor KBPKNScalingFunction::apply(const Position& pos) {
 /// If the pawn is a rook pawn on the 7th rank and the defending king prevents
 /// the pawn from advancing, the position is drawn.
 
-ScaleFactor KNPKScalingFunction::apply(const Position& pos) {
+ScaleFactor KNPKScalingFunction::apply(const Position &pos) {
   assert(pos.non_pawn_material(strongerSide) == KnightValueMidgame);
   assert(pos.knight_count(strongerSide) == 1);
   assert(pos.pawn_count(strongerSide) == 1);
@@ -794,6 +843,55 @@ ScaleFactor KNPKScalingFunction::apply(const Position& pos) {
 }
 
 
+/// KPKPScalingFunction scales KP vs KP endgames.  This is done by removing
+/// the weakest side's pawn and probing the KP vs K bitbase:  If the weakest
+/// side has a draw without the pawn, she probably has at least a draw with
+/// the pawn as well.  The exception is when the stronger side's pawn is far
+/// advanced and not on a rook file; in this case it is often possible to win
+/// (e.g. 8/4k3/3p4/3P4/6K1/8/8/8 w - - 0 1).
+
+ScaleFactor KPKPScalingFunction::apply(const Position &pos) {
+  assert(pos.non_pawn_material(strongerSide) == Value(0));
+  assert(pos.non_pawn_material(weakerSide) == Value(0));
+  assert(pos.pawn_count(WHITE) == 1);
+  assert(pos.pawn_count(BLACK) == 1);
+
+  Square wksq, bksq, wpsq;
+  Color stm;
+
+  if(strongerSide == WHITE) {
+    wksq = pos.king_square(WHITE);
+    bksq = pos.king_square(BLACK);
+    wpsq = pos.pawn_list(WHITE, 0);
+    stm = pos.side_to_move();
+  }
+  else {
+    wksq = flip_square(pos.king_square(BLACK));
+    bksq = flip_square(pos.king_square(WHITE));
+    wpsq = flip_square(pos.pawn_list(BLACK, 0));
+    stm = opposite_color(pos.side_to_move());
+  }
+
+  if(square_file(wpsq) >= FILE_E) {
+    wksq = flop_square(wksq);
+    bksq = flop_square(bksq);
+    wpsq = flop_square(wpsq);
+  }
+
+  // If the pawn has advanced to the fifth rank or further, and is not a
+  // rook pawn, it's too dangerous to assume that it's at least a draw.
+  if(square_rank(wpsq) >= RANK_5 && square_file(wpsq) != FILE_A)
+    return SCALE_FACTOR_NONE;
+
+  // Probe the KPK bitbase with the weakest side's pawn removed.  If it's a
+  // draw, it's probably at least a draw even with the pawn.
+  if(probe_kpk(wksq, wpsq, bksq, stm))
+    return SCALE_FACTOR_NONE;
+  else
+    return ScaleFactor(0);
+}
+
+
 /// init_bitbases() is called during program initialization, and simply loads
 /// bitbases from disk into memory.  At the moment, there is only the bitbase
 /// for KP vs K, but we may decide to add other bitbases later.
