@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@
 //// Prototypes
 ////
 
-extern uint32 genrand_int32(void);
-extern uint64 genrand_int64(void);
+extern uint32_t genrand_int32(void);
+extern uint64_t genrand_int64(void);
 extern void init_mersenne(void);
 
 
