@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 /// Version number.  If this is left empty, the current date (in the format
 /// YYMMDD) is used as a version number.
 
-const std::string EngineVersion = "2.0.1";
+const std::string EngineVersion = "2.2";
 
 
 ////
@@ -52,7 +52,6 @@ const std::string EngineVersion = "2.0.1";
 ////
 
 extern const std::string engine_name();
-extern const std::string int_to_string(int i);
 extern int get_system_time();
 extern int cpu_count();
 extern int Bioskey();
