@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 
 #include <cassert>
 
-#include "distance.h"
 #include "movegen.h"
 
 
@@ -33,21 +32,21 @@
 
 namespace {
   
-  int generate_white_pawn_captures(const Position& pos, MoveStack* mlist);
-  int generate_black_pawn_captures(const Position& pos, MoveStack* mlist);
-  int generate_white_pawn_noncaptures(const Position& pos, MoveStack* mlist);
-  int generate_black_pawn_noncaptures(const Position& pos, MoveStack* mlist);
-  int generate_knight_moves(const Position& pos, MoveStack* mlist, 
+  int generate_white_pawn_captures(const Position &pos, MoveStack *mlist);
+  int generate_black_pawn_captures(const Position &pos, MoveStack *mlist);
+  int generate_white_pawn_noncaptures(const Position &pos, MoveStack *mlist);
+  int generate_black_pawn_noncaptures(const Position &pos, MoveStack *mlist);
+  int generate_knight_moves(const Position &pos, MoveStack *mlist, 
                             Color side, Bitboard target);
-  int generate_bishop_moves(const Position& pos, MoveStack* mlist, 
+  int generate_bishop_moves(const Position &pos, MoveStack *mlist, 
                             Color side, Bitboard target);
-  int generate_rook_moves(const Position& pos, MoveStack* mlist, 
+  int generate_rook_moves(const Position &pos, MoveStack *mlist, 
                           Color side, Bitboard target);
-  int generate_queen_moves(const Position& pos, MoveStack* mlist, 
+  int generate_queen_moves(const Position &pos, MoveStack *mlist, 
                            Color side, Bitboard target);
-  int generate_king_moves(const Position& pos, MoveStack* mlist, 
+  int generate_king_moves(const Position &pos, MoveStack *mlist, 
                           Square from, Bitboard target);
-  int generate_castle_moves(const Position& pos, MoveStack* mlist, Color us);
+  int generate_castle_moves(const Position &pos, MoveStack *mlist, Color us);
 
 }
 
@@ -60,7 +59,7 @@ namespace {
 /// generate_captures generates() all pseudo-legal captures and queen
 /// promotions.  The return value is the number of moves generated.
 
-int generate_captures(const Position& pos, MoveStack* mlist) {
+int generate_captures(const Position &pos, MoveStack *mlist) {
   Color us = pos.side_to_move();
   Bitboard target = pos.pieces_of_color(opposite_color(us));
   int n = 0;
@@ -85,7 +84,7 @@ int generate_captures(const Position& pos, MoveStack* mlist) {
 /// generate_noncaptures() generates all pseudo-legal non-captures and 
 /// underpromotions.  The return value is the number of moves generated.
 
-int generate_noncaptures(const Position& pos, MoveStack* mlist) {
+int generate_noncaptures(const Position &pos, MoveStack *mlist) {
   Color us = pos.side_to_move();
   Bitboard target = pos.empty_squares();
   int n = 0;
@@ -112,7 +111,7 @@ int generate_noncaptures(const Position& pos, MoveStack* mlist) {
 /// checks, except castling moves (will add this later).  It returns the
 /// number of generated moves.  
 
-int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
+int generate_checks(const Position &pos, MoveStack *mlist, Bitboard dc) {
   Color us, them;
   Square ksq, from, to;
   Bitboard empty, checkSqs, b1, b2, b3;
@@ -139,7 +138,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
     // generate captures.
 
     // Find all friendly pawns not on the enemy king's file:
-    b1 = pos.pawns_of_color(us) & ~file_bb(ksq);
+    b1 = pos.pawns(us) & ~file_bb(ksq);
 
     // Discovered checks, single pawn pushes:
     b2 = b3 = ((b1 & dc) << 8) & ~Rank8BB & empty;
@@ -182,7 +181,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
     // generate captures.
 
     // Find all friendly pawns not on the enemy king's file:
-    b1 = pos.pawns_of_color(us) & ~file_bb(ksq);
+    b1 = pos.pawns(us) & ~file_bb(ksq);
 
     // Discovered checks, single pawn pushes:
     b2 = b3 = ((b1 & dc) >> 8) & ~Rank1BB & empty;
@@ -212,7 +211,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
     }
 
     // Direct checks, double pawn pushes:
-    b3 = ((b2 & Rank6BB) >> 8) & empty & pos.black_pawn_attacks(ksq);
+    b3 = ((b2 & Rank6BB) >> 8) & empty & pos.white_pawn_attacks(ksq);
     while(b3) {
       to = pop_1st_bit(&b3);
       mlist[n++].move = make_move(to - DELTA_S - DELTA_S, to);
@@ -220,7 +219,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
   }
 
   // Knight moves
-  b1 = pos.knights_of_color(us);
+  b1 = pos.knights(us);
   if(b1) {
     // Discovered knight checks:
     b2 = b1 & dc;
@@ -247,7 +246,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
   }
 
   // Bishop moves
-  b1 = pos.bishops_of_color(us);
+  b1 = pos.bishops(us);
   if(b1) {
     // Discovered bishop checks:
     b2 = b1 & dc;
@@ -274,7 +273,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
   }
 
   // Rook moves
-  b1 = pos.rooks_of_color(us);
+  b1 = pos.rooks(us);
   if(b1) {
     // Discovered rook checks:
     b2 = b1 & dc;
@@ -301,7 +300,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
   }
 
   // Queen moves
-  b1 = pos.queens_of_color(us);
+  b1 = pos.queens(us);
   if(b1) {
     // Discovered queen checks are impossible!
 
@@ -338,7 +337,7 @@ int generate_checks(const Position& pos, MoveStack* mlist, Bitboard dc) {
 /// only legal moves.  It returns the number of generated moves.  This
 /// function is very ugly, and needs cleaning up some time later.  FIXME
 
-int generate_evasions(const Position& pos, MoveStack* mlist) {
+int generate_evasions(const Position &pos, MoveStack *mlist) {
   Color us, them;
   Bitboard checkers = pos.checkers();
   Bitboard pinned, b1, b2;
@@ -366,13 +365,12 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
     // the low-level bishop_attacks_bb and rook_attacks_bb with the bitboard
     // b2 (the occupied squares with the king removed) in order to test whether
     // the king will remain in check on the destination square.
-    if(((pos.pawn_attacks(us, to)&pos.pawns_of_color(them)) == EmptyBoardBB) &&
-       ((pos.knight_attacks(to)&pos.knights_of_color(them)) == EmptyBoardBB) &&
-       ((pos.king_attacks(to)&pos.kings_of_color(them)) == EmptyBoardBB) &&
-       ((bishop_attacks_bb(to, b2) & pos.bishops_and_queens_of_color(them))
+    if(((pos.pawn_attacks(us, to) & pos.pawns(them)) == EmptyBoardBB) &&
+       ((pos.knight_attacks(to) & pos.knights(them)) == EmptyBoardBB) &&
+       ((pos.king_attacks(to) & pos.kings(them)) == EmptyBoardBB) &&
+       ((bishop_attacks_bb(to, b2) & pos.bishops_and_queens(them))
         == EmptyBoardBB) &&
-       ((rook_attacks_bb(to, b2) & pos.rooks_and_queens_of_color(them))
-        == EmptyBoardBB))
+       ((rook_attacks_bb(to, b2) & pos.rooks_and_queens(them)) == EmptyBoardBB))
       mlist[n++].move = make_move(ksq, to);
   }
 
@@ -390,7 +388,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
     // Generate captures of the checking piece:
 
     // Pawn captures:
-    b1 = pos.pawn_attacks(them, checksq) & pos.pawns_of_color(us) & ~pinned;
+    b1 = pos.pawn_attacks(them, checksq) & pos.pawns(us) & ~pinned;
     while(b1) {
       from = pop_1st_bit(&b1);
       if(pawn_rank(us, checksq) == RANK_8) {
@@ -404,14 +402,14 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
     }
 
     // Knight captures:
-    b1 = pos.knight_attacks(checksq) & pos.knights_of_color(us) & ~pinned;
+    b1 = pos.knight_attacks(checksq) & pos.knights(us) & ~pinned;
     while(b1) {
       from = pop_1st_bit(&b1);
       mlist[n++].move = make_move(from, checksq);
     }
 
     // Bishop and queen captures:
-    b1 = pos.bishop_attacks(checksq) & pos.bishops_and_queens_of_color(us)
+    b1 = pos.bishop_attacks(checksq) & pos.bishops_and_queens(us)
       & ~pinned;
     while(b1) {
       from = pop_1st_bit(&b1);
@@ -419,7 +417,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
     }
 
     // Rook and queen captures:
-    b1 = pos.rook_attacks(checksq) & pos.rooks_and_queens_of_color(us)
+    b1 = pos.rook_attacks(checksq) & pos.rooks_and_queens(us)
       & ~pinned;
     while(b1) {
       from = pop_1st_bit(&b1);
@@ -437,7 +435,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
       // ugly, and uses separate clauses for white and black pawns. :-(
       if(us == WHITE) {
         // Find non-pinned pawns:
-        b1 = pos.pawns_of_color(WHITE) & ~pinned;
+        b1 = pos.pawns(WHITE) & ~pinned;
 
         // Single pawn pushes.  We don't have to AND with empty squares here,
         // because the blocking squares will always be empty.
@@ -465,7 +463,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
       }
       else { // (us == BLACK)
         // Find non-pinned pawns:
-        b1 = pos.pawns_of_color(BLACK) & ~pinned;
+        b1 = pos.pawns(BLACK) & ~pinned;
 
         // Single pawn pushes.  We don't have to AND with empty squares here,
         // because the blocking squares will always be empty.
@@ -493,7 +491,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
       }
 
       // Knight moves
-      b1 = pos.knights_of_color(us) & ~pinned;
+      b1 = pos.knights(us) & ~pinned;
       while(b1) {
         from = pop_1st_bit(&b1);
         b2 = pos.knight_attacks(from) & blockSquares;
@@ -504,7 +502,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
       }
       
       // Bishop moves
-      b1 = pos.bishops_of_color(us) & ~pinned;
+      b1 = pos.bishops(us) & ~pinned;
       while(b1) {
         from = pop_1st_bit(&b1);
         b2 = pos.bishop_attacks(from) & blockSquares;
@@ -515,7 +513,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
       }
       
       // Rook moves
-      b1 = pos.rooks_of_color(us) & ~pinned;
+      b1 = pos.rooks(us) & ~pinned;
       while(b1) {
         from = pop_1st_bit(&b1);
         b2 = pos.rook_attacks(from) & blockSquares;
@@ -526,7 +524,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
       }
       
       // Queen moves
-      b1 = pos.queens_of_color(us) & ~pinned;
+      b1 = pos.queens(us) & ~pinned;
       while(b1) {
         from = pop_1st_bit(&b1);
         b2 = pos.queen_attacks(from) & blockSquares;
@@ -542,9 +540,9 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
     // check.  If pos.ep_square() is set, the last move made must have been
     // a double pawn push.  If, furthermore, the checking piece is a pawn,
     // an en passant check evasion may be possible.
-    if(pos.ep_square() != SQ_NONE && (checkers & pos.pawns_of_color(them))) {
+    if(pos.ep_square() != SQ_NONE && (checkers & pos.pawns(them))) {
       to = pos.ep_square();
-      b1 = pos.pawn_attacks(them, to) & pos.pawns_of_color(us);
+      b1 = pos.pawn_attacks(them, to) & pos.pawns(us);
       assert(b1 != EmptyBoardBB);
       b1 &= ~pinned;
       while(b1) {
@@ -558,9 +556,9 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
         b2 = pos.occupied_squares();
         clear_bit(&b2, from);
         clear_bit(&b2, checksq);
-        if(((bishop_attacks_bb(ksq, b2) & pos.bishops_and_queens_of_color(them))
+        if(((bishop_attacks_bb(ksq, b2) & pos.bishops_and_queens(them))
             == EmptyBoardBB) &&
-           ((rook_attacks_bb(ksq, b2) & pos.rooks_and_queens_of_color(them))
+           ((rook_attacks_bb(ksq, b2) & pos.rooks_and_queens(them))
             == EmptyBoardBB))
           mlist[n++].move = make_ep_move(from, to);
       }
@@ -577,7 +575,7 @@ int generate_evasions(const Position& pos, MoveStack* mlist) {
 /// very hard to write an efficient legal move generator, but for the moment
 /// we don't need it.
 
-int generate_legal_moves(const Position& pos, MoveStack* mlist) {
+int generate_legal_moves(const Position &pos, MoveStack *mlist) {
   assert(pos.is_ok());
 
   if(pos.is_check())
@@ -607,7 +605,7 @@ int generate_legal_moves(const Position& pos, MoveStack* mlist) {
 /// returned.  If not, the function returns MOVE_NONE.  This function must
 /// only be used when the side to move is not in check.
 
-Move generate_move_if_legal(const Position& pos, Move m, Bitboard pinned) {
+Move generate_move_if_legal(const Position &pos, Move m, Bitboard pinned) {
   Color us, them;
   Square from, to;
   Piece pc;
@@ -862,8 +860,8 @@ Move generate_move_if_legal(const Position& pos, Move m, Bitboard pinned) {
 
 namespace {
 
-  int generate_white_pawn_captures(const Position& pos, MoveStack* mlist) {
-    Bitboard pawns = pos.pawns_of_color(WHITE);
+  int generate_white_pawn_captures(const Position &pos, MoveStack *mlist) {
+    Bitboard pawns = pos.pawns(WHITE);
     Bitboard enemyPieces = pos.pieces_of_color(BLACK);
     Bitboard b1, b2;
     Square sq;
@@ -925,8 +923,8 @@ namespace {
   }
 
 
-  int generate_black_pawn_captures(const Position& pos, MoveStack* mlist) {
-    Bitboard pawns = pos.pawns_of_color(BLACK);
+  int generate_black_pawn_captures(const Position &pos, MoveStack *mlist) {
+    Bitboard pawns = pos.pawns(BLACK);
     Bitboard enemyPieces = pos.pieces_of_color(WHITE);
     Bitboard b1, b2;
     Square sq;
@@ -988,8 +986,8 @@ namespace {
   }
     
 
-  int generate_white_pawn_noncaptures(const Position& pos, MoveStack* mlist) {
-    Bitboard pawns = pos.pawns_of_color(WHITE);
+  int generate_white_pawn_noncaptures(const Position &pos, MoveStack *mlist) {
+    Bitboard pawns = pos.pawns(WHITE);
     Bitboard enemyPieces = pos.pieces_of_color(BLACK);
     Bitboard emptySquares = pos.empty_squares();
     Bitboard b1, b2;
@@ -1040,8 +1038,8 @@ namespace {
   }
 
 
-  int generate_black_pawn_noncaptures(const Position& pos, MoveStack* mlist) {
-    Bitboard pawns = pos.pawns_of_color(BLACK);
+  int generate_black_pawn_noncaptures(const Position &pos, MoveStack *mlist) {
+    Bitboard pawns = pos.pawns(BLACK);
     Bitboard enemyPieces = pos.pieces_of_color(WHITE);
     Bitboard emptySquares = pos.empty_squares();
     Bitboard b1, b2;
@@ -1092,7 +1090,7 @@ namespace {
   }
 
   
-  int generate_knight_moves(const Position& pos, MoveStack* mlist, 
+  int generate_knight_moves(const Position &pos, MoveStack *mlist, 
                             Color side, Bitboard target) {
     Square from, to;
     Bitboard b;
@@ -1110,7 +1108,7 @@ namespace {
   }
 
 
-  int generate_bishop_moves(const Position& pos, MoveStack* mlist, 
+  int generate_bishop_moves(const Position &pos, MoveStack *mlist, 
                             Color side, Bitboard target) {
     Square from, to;
     Bitboard b;
@@ -1128,7 +1126,7 @@ namespace {
   }
 
 
-  int generate_rook_moves(const Position& pos, MoveStack* mlist, 
+  int generate_rook_moves(const Position &pos, MoveStack *mlist, 
                           Color side, Bitboard target) {
     Square from, to;
     Bitboard b;
@@ -1146,7 +1144,7 @@ namespace {
   }
 
 
-  int generate_queen_moves(const Position& pos, MoveStack* mlist, 
+  int generate_queen_moves(const Position &pos, MoveStack *mlist, 
                            Color side, Bitboard target) {
     Square from, to;
     Bitboard b;
@@ -1164,7 +1162,7 @@ namespace {
   }
 
 
-  int generate_king_moves(const Position& pos, MoveStack* mlist, 
+  int generate_king_moves(const Position &pos, MoveStack *mlist, 
                           Square from, Bitboard target) {
     Square to;
     Bitboard b;
@@ -1179,7 +1177,7 @@ namespace {
   }
 
 
-  int generate_castle_moves(const Position& pos, MoveStack* mlist, Color us) {
+  int generate_castle_moves(const Position &pos, MoveStack *mlist, Color us) {
     int n = 0;
 
     if(pos.can_castle(us)) {
@@ -1197,11 +1195,11 @@ namespace {
         assert(pos.piece_on(rsq) == rook_of_color(us));
 
         for(s = Min(ksq, g1); s <= Max(ksq, g1); s++)
-          if((s != ksq && s != rsq && pos.piece_on(s) != EMPTY) ||
-             pos.square_is_attacked(s, them))
+          if((s != ksq && s != rsq && pos.square_is_occupied(s))
+             || pos.square_is_attacked(s, them))
             illegal = true;
         for(s = Min(rsq, f1); s <= Max(rsq, f1); s++)
-          if(s != ksq && s != rsq && pos.piece_on(s) != EMPTY)
+          if(s != ksq && s != rsq && pos.square_is_occupied(s))
             illegal = true;
 
         if(!illegal)
@@ -1218,11 +1216,11 @@ namespace {
         assert(pos.piece_on(rsq) == rook_of_color(us));
 
         for(s = Min(ksq, c1); s <= Max(ksq, c1); s++)
-          if((s != ksq && s != rsq && pos.piece_on(s) != EMPTY) ||
-             pos.square_is_attacked(s, them))
+          if((s != ksq && s != rsq && pos.square_is_occupied(s))
+             || pos.square_is_attacked(s, them))
             illegal = true;
         for(s = Min(rsq, d1); s <= Max(rsq, d1); s++)
-          if(s != ksq && s != rsq && pos.piece_on(s) != EMPTY)
+          if(s != ksq && s != rsq && pos.square_is_occupied(s))
             illegal = true;
         if(square_file(rsq) == FILE_B &&
            (pos.piece_on(relative_square(us, SQ_A1)) == rook_of_color(them) ||
