@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,20 +23,23 @@
 
 #include <cassert>
 #include <cstdio>
+#include <cstring>
 #include <iostream>
 #include <fstream>
 
 #include "mersenne.h"
+#include "movegen.h"
 #include "movepick.h"
 #include "position.h"
 #include "psqtab.h"
+#include "ucioption.h"
 
 
 ////
 //// Variables
 ////
 
-CastleRights Position::castleRightsMask[64];
+int Position::castleRightsMask[64];
 
 Key Position::zobrist[2][8][64];
 Key Position::zobEp[64];
@@ -44,6 +47,9 @@ Key Position::zobCastle[16];
 Key Position::zobMaterial[2][8][16];
 Key Position::zobSideToMove;
 
+Value Position::MgPieceSquareTable[16][64];
+Value Position::EgPieceSquareTable[16][64];
+
 
 ////
 //// Functions
@@ -53,11 +59,11 @@ Key Position::zobSideToMove;
 
 Position::Position() { }  // Do we really need this one?
 
-Position::Position(const Position& pos) {
+Position::Position(const Position &pos) {
   this->copy(pos);
 }
 
-Position::Position(const std::string& fen) {
+Position::Position(const std::string &fen) {
   this->from_fen(fen);
 }
 
@@ -66,7 +72,7 @@ Position::Position(const std::string& fen) {
 /// string. This function is not very robust - make sure that input FENs are
 /// correct (this is assumed to be the responsibility of the GUI).
 
-void Position::from_fen(const std::string& fen) {
+void Position::from_fen(const std::string &fen) {
   File file;
   Rank rank;
   int i;
@@ -223,7 +229,7 @@ const std::string Position::to_fen() const {
     skip = 0;
     for(File file = FILE_A; file <= FILE_H; file++) {
       Square square = make_square(file, rank);
-      if(this->piece_on(square) != EMPTY) {
+      if(this->square_is_occupied(square)) {
         if(skip > 0) result += (char)skip + '0';
         result += pieceLetters[this->piece_on(square)];
         skip = 0;
@@ -252,9 +258,36 @@ const std::string Position::to_fen() const {
 }
 
 
+/// Position::print() prints an ASCII representation of the position to
+/// the standard output.
+
+void Position::print() const {
+  char pieceStrings[][8] = 
+    {"| ? ", "| P ", "| N ", "| B ", "| R ", "| Q ", "| K ", "| ? ", 
+     "| ? ", "|=P=", "|=N=", "|=B=", "|=R=", "|=Q=", "|=K="
+    };
+
+  for(Rank rank = RANK_8; rank >= RANK_1; rank--) {
+    std::cout << "+---+---+---+---+---+---+---+---+\n";
+    for(File file = FILE_A; file <= FILE_H; file++) {
+      Square sq = make_square(file, rank);
+      Piece piece = this->piece_on(sq);
+      if(piece == EMPTY)
+        std::cout << ((square_color(sq) == WHITE)? "|   " : "| . ");
+      else
+        std::cout << pieceStrings[piece];
+    }
+    std::cout << "|\n";
+  }
+  std::cout << "+---+---+---+---+---+---+---+---+\n"; 
+  std::cout << this->to_fen() << std::endl;
+  std::cout << key << std::endl;
+}
+
+
 /// Position::copy() creates a copy of the input position.
 
-void Position::copy(const Position& pos) {
+void Position::copy(const Position &pos) {
   memcpy(this, &pos, sizeof(Position));
 }
 
@@ -270,7 +303,7 @@ Bitboard Position::pinned_pieces(Color c) const {
   pinned = EmptyBoardBB;
   b1 = this->occupied_squares();
 
-  sliders = this->rooks_and_queens_of_color(them) & ~this->checkers();
+  sliders = this->rooks_and_queens(them) & ~this->checkers();
   if(sliders & RookPseudoAttacks[ksq]) {
     b2 = this->rook_attacks(ksq) & this->pieces_of_color(c);
     pinners = rook_attacks_bb(ksq, b1 ^ b2) & sliders;
@@ -280,7 +313,7 @@ Bitboard Position::pinned_pieces(Color c) const {
     }
   }
 
-  sliders = this->bishops_and_queens_of_color(them) & ~this->checkers();
+  sliders = this->bishops_and_queens(them) & ~this->checkers();
   if(sliders & BishopPseudoAttacks[ksq]) {
     b2 = this->bishop_attacks(ksq) & this->pieces_of_color(c);
     pinners = bishop_attacks_bb(ksq, b1 ^ b2) & sliders;
@@ -293,6 +326,7 @@ Bitboard Position::pinned_pieces(Color c) const {
   return pinned;
 }
 
+
 /// Position:discovered_check_candidates() returns a bitboard containing all
 /// pieces for the given side which are candidates for giving a discovered
 /// check.  The code is almost the same as the function for finding pinned
@@ -305,7 +339,7 @@ Bitboard Position::discovered_check_candidates(Color c) const {
   dc = EmptyBoardBB;
   b1 = this->occupied_squares();
 
-  sliders = this->rooks_and_queens_of_color(c);
+  sliders = this->rooks_and_queens(c);
   if(sliders & RookPseudoAttacks[ksq]) {
     b2 = this->rook_attacks(ksq) & this->pieces_of_color(c);
     checkers = rook_attacks_bb(ksq, b1 ^ b2) & sliders;
@@ -315,7 +349,7 @@ Bitboard Position::discovered_check_candidates(Color c) const {
     }
   }
   
-  sliders = this->bishops_and_queens_of_color(c);
+  sliders = this->bishops_and_queens(c);
   if(sliders & BishopPseudoAttacks[ksq]) {
     b2 = this->bishop_attacks(ksq) & this->pieces_of_color(c);
     checkers = bishop_attacks_bb(ksq, b1 ^ b2) & sliders;
@@ -334,11 +368,11 @@ Bitboard Position::discovered_check_candidates(Color c) const {
 
 bool Position::square_is_attacked(Square s, Color c) const {
   return
-    (this->pawn_attacks(opposite_color(c), s) & this->pawns_of_color(c)) ||
-    (this->knight_attacks(s) & this->knights_of_color(c)) ||
-    (this->king_attacks(s) & this->kings_of_color(c)) ||
-    (this->rook_attacks(s) & this->rooks_and_queens_of_color(c)) ||
-    (this->bishop_attacks(s) & this->bishops_and_queens_of_color(c));
+    (this->pawn_attacks(opposite_color(c), s) & this->pawns(c)) ||
+    (this->knight_attacks(s) & this->knights(c)) ||
+    (this->king_attacks(s) & this->kings(c)) ||
+    (this->rook_attacks(s) & this->rooks_and_queens(c)) ||
+    (this->bishop_attacks(s) & this->bishops_and_queens(c));
 }
 
 
@@ -349,8 +383,8 @@ bool Position::square_is_attacked(Square s, Color c) const {
 
 Bitboard Position::attacks_to(Square s) const {
   return
-    (this->black_pawn_attacks(s) & this->pawns_of_color(WHITE)) |
-    (this->white_pawn_attacks(s) & this->pawns_of_color(BLACK)) |
+    (this->black_pawn_attacks(s) & this->pawns(WHITE)) |
+    (this->white_pawn_attacks(s) & this->pawns(BLACK)) |
     (this->knight_attacks(s) & this->pieces_of_type(KNIGHT)) |
     (this->rook_attacks(s) & this->rooks_and_queens()) |
     (this->bishop_attacks(s) & this->bishops_and_queens()) |
@@ -446,8 +480,8 @@ bool Position::move_is_legal(Move m, Bitboard pinned) const {
 
     clear_bit(&b, from); clear_bit(&b, capsq); set_bit(&b, to);
     return
-      (!(rook_attacks_bb(ksq, b) & this->rooks_and_queens_of_color(them)) &&
-       !(bishop_attacks_bb(ksq, b) & this->bishops_and_queens_of_color(them)));
+      (!(rook_attacks_bb(ksq, b) & this->rooks_and_queens(them)) &&
+       !(bishop_attacks_bb(ksq, b) & this->bishops_and_queens(them)));
   }
   
   // If the moving piece is a king, check whether the destination 
@@ -534,8 +568,8 @@ bool Position::move_is_check(Move m, Bitboard dcCandidates) const {
 
       clear_bit(&b, from); clear_bit(&b, capsq); set_bit(&b, to);
       return
-        ((rook_attacks_bb(ksq, b) & this->rooks_and_queens_of_color(us)) || 
-         (bishop_attacks_bb(ksq, b) & this->bishops_and_queens_of_color(us)));
+        ((rook_attacks_bb(ksq, b) & this->rooks_and_queens(us)) || 
+         (bishop_attacks_bb(ksq, b) & this->bishops_and_queens(us)));
     }
     return false;
 
@@ -609,7 +643,7 @@ bool Position::move_is_check(Move m, Bitboard dcCandidates) const {
 }
 
 
-/// Position::move_is_capture() tests() whether a move from the current
+/// Position::move_is_capture() tests whether a move from the current
 /// position is a capture.
 
 bool Position::move_is_capture(Move m) const {
@@ -619,6 +653,34 @@ bool Position::move_is_capture(Move m) const {
 }
 
 
+/// Position::move_attacks_square() tests whether a move from the current
+/// position attacks a given square.  Only attacks by the moving piece are
+/// considered; the function does not handle X-ray attacks.
+
+bool Position::move_attacks_square(Move m, Square s) const {
+  assert(move_is_ok(m));
+  assert(square_is_ok(s));
+
+  Square f = move_from(m), t = move_to(m);
+
+  assert(this->square_is_occupied(f));
+
+  switch(this->piece_on(f)) {
+  case WP: return this->white_pawn_attacks_square(t, s);
+  case BP: return this->black_pawn_attacks_square(t, s);
+  case WN: case BN: return this->knight_attacks_square(t, s);
+  case WB: case BB: return this->bishop_attacks_square(t, s);
+  case WR: case BR: return this->rook_attacks_square(t, s);
+  case WQ: case BQ: return this->queen_attacks_square(t, s);
+  case WK: case BK: return this->king_attacks_square(t, s);
+  default: assert(false);
+  }
+
+  return false;
+}
+
+
+
 /// Position::backup() is called when making a move.  All information 
 /// necessary to restore the position when the move is later unmade
 /// is saved to an UndoInfo object.  The function Position::restore
@@ -626,7 +688,7 @@ bool Position::move_is_capture(Move m) const {
 /// a restore with the same UndoInfo object, the position is restored
 /// to the state before backup was called.
 
-void Position::backup(UndoInfo& u) const {
+void Position::backup(UndoInfo &u) const {
   u.castleRights = castleRights;
   u.epSquare = epSquare;
   u.checkersBB = checkersBB;
@@ -644,7 +706,7 @@ void Position::backup(UndoInfo& u) const {
 /// Position::restore() is called when unmaking a move.  It copies back
 /// the information backed up during a previous call to Position::backup.
 
-void Position::restore(const UndoInfo& u) {
+void Position::restore(const UndoInfo &u) {
   castleRights = u.castleRights;
   epSquare = u.epSquare;
   checkersBB = u.checkersBB;
@@ -667,11 +729,11 @@ void Position::restore(const UndoInfo& u) {
 /// the discovered check candidates makes it easier to update the checkersBB
 /// member variable in the position object.
 
-void Position::do_move(Move m, UndoInfo& u) {
+void Position::do_move(Move m, UndoInfo &u) {
   this->do_move(m, u, this->discovered_check_candidates(this->side_to_move()));
 }
 
-void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
+void Position::do_move(Move m, UndoInfo &u, Bitboard dcCandidates) {
   assert(this->is_ok());
   assert(move_is_ok(m));
 
@@ -784,9 +846,9 @@ void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
     if(piece == PAWN) {
       if(abs(int(to) - int(from)) == 16) {
         if((us == WHITE && (this->white_pawn_attacks(from + DELTA_N) &
-                            this->pawns_of_color(BLACK))) ||
+                            this->pawns(BLACK))) ||
            (us == BLACK && (this->black_pawn_attacks(from + DELTA_S) &
-                            this->pawns_of_color(WHITE)))) {
+                            this->pawns(WHITE)))) {
           epSquare = Square((int(from) + int(to)) / 2);
           key ^= zobEp[epSquare];
         }
@@ -818,8 +880,8 @@ void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
         set_bit(&checkersBB, to);
       if(bit_is_set(dcCandidates, from))
         checkersBB |=
-          ((this->rook_attacks(ksq) & this->rooks_and_queens_of_color(us)) |
-           (this->bishop_attacks(ksq) & this->bishops_and_queens_of_color(us)));
+          ((this->rook_attacks(ksq) & this->rooks_and_queens(us)) |
+           (this->bishop_attacks(ksq) & this->bishops_and_queens(us)));
       break;
 
     case KNIGHT:
@@ -827,8 +889,8 @@ void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
         set_bit(&checkersBB, to);
       if(bit_is_set(dcCandidates, from))
         checkersBB |=
-          ((this->rook_attacks(ksq) & this->rooks_and_queens_of_color(us)) |
-           (this->bishop_attacks(ksq) & this->bishops_and_queens_of_color(us)));
+          ((this->rook_attacks(ksq) & this->rooks_and_queens(us)) |
+           (this->bishop_attacks(ksq) & this->bishops_and_queens(us)));
       break;
 
     case BISHOP:
@@ -836,7 +898,7 @@ void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
         set_bit(&checkersBB, to);
       if(bit_is_set(dcCandidates, from))
         checkersBB |=
-          (this->rook_attacks(ksq) & this->rooks_and_queens_of_color(us));
+          (this->rook_attacks(ksq) & this->rooks_and_queens(us));
       break;
 
     case ROOK:
@@ -844,7 +906,7 @@ void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
         set_bit(&checkersBB, to);
       if(bit_is_set(dcCandidates, from))
         checkersBB |=
-          (this->bishop_attacks(ksq) & this->bishops_and_queens_of_color(us));
+          (this->bishop_attacks(ksq) & this->bishops_and_queens(us));
       break;
 
     case QUEEN:
@@ -855,8 +917,8 @@ void Position::do_move(Move m, UndoInfo& u, Bitboard dcCandidates) {
     case KING:
       if(bit_is_set(dcCandidates, from))
         checkersBB |=
-          ((this->rook_attacks(ksq) & this->rooks_and_queens_of_color(us)) |
-           (this->bishop_attacks(ksq) & this->bishops_and_queens_of_color(us)));
+          ((this->rook_attacks(ksq) & this->rooks_and_queens(us)) |
+           (this->bishop_attacks(ksq) & this->bishops_and_queens(us)));
       break;
 
     default:
@@ -979,7 +1041,7 @@ void Position::do_castle_move(Move m) {
 /// UndoInfo object, which has been initialized in Position::do_move, is
 /// used to store the captured piece (if any).
 
-void Position::do_promotion_move(Move m, UndoInfo& u) {
+void Position::do_promotion_move(Move m, UndoInfo &u) {
   Color us, them;
   Square from, to;
   PieceType capture, promotion;
@@ -1189,7 +1251,7 @@ void Position::do_ep_move(Move m) {
 /// important that Position::undo_move is called with the same move and UndoInfo
 /// object as the earlier call to Position::do_move.
 
-void Position::undo_move(Move m, const UndoInfo& u) {
+void Position::undo_move(Move m, const UndoInfo &u) {
   assert(this->is_ok());
   assert(move_is_ok(m));
 
@@ -1342,7 +1404,7 @@ void Position::undo_castle_move(Move m) {
 /// function.  The UndoInfo object, which has been initialized in
 /// Position::do_move, is used to put back the captured piece (if any).
 
-void Position::undo_promotion_move(Move m, const UndoInfo& u) {
+void Position::undo_promotion_move(Move m, const UndoInfo &u) {
   Color us, them;
   Square from, to;
   PieceType capture, promotion;
@@ -1358,14 +1420,13 @@ void Position::undo_promotion_move(Move m, const UndoInfo& u) {
 
   from = move_from(m);
   to = move_to(m);
-  promotion = move_promotion(m);
 
   assert(pawn_rank(us, to) == RANK_8);
-  assert(this->piece_on(to)==piece_of_color_and_type(us, promotion));
   assert(this->piece_on(from) == EMPTY);
 
   // Remove promoted piece:
   promotion = move_promotion(m);
+  assert(this->piece_on(to)==piece_of_color_and_type(us, promotion));
   assert(promotion >= KNIGHT && promotion <= QUEEN);
   clear_bit(&(byColorBB[us]), to);
   clear_bit(&(byTypeBB[promotion]), to);
@@ -1459,7 +1520,7 @@ void Position::undo_ep_move(Move m) {
   clear_bit(&(byTypeBB[0]), to);
   board[to] = EMPTY;
 
-  // Replace moving piece at destination square:
+  // Replace moving piece at source square:
   set_bit(&(byColorBB[us]), from);
   set_bit(&(byTypeBB[PAWN]), from);
   set_bit(&(byTypeBB[0]), from);
@@ -1479,7 +1540,7 @@ void Position::undo_ep_move(Move m) {
 /// Position::do_null_move makes() a "null move": It switches the side to move
 /// and updates the hash key without executing any move on the board.
 
-void Position::do_null_move(UndoInfo& u) {
+void Position::do_null_move(UndoInfo &u) {
   assert(this->is_ok());
   assert(!this->is_check());
   
@@ -1511,7 +1572,7 @@ void Position::do_null_move(UndoInfo& u) {
 
 /// Position::undo_null_move() unmakes a "null move".
 
-void Position::undo_null_move(const UndoInfo& u) {
+void Position::undo_null_move(const UndoInfo &u) {
   assert(this->is_ok());
   assert(!this->is_check());
   
@@ -1569,8 +1630,8 @@ int Position::see(Square from, Square to) const {
     (bishop_attacks_bb(to, occ) & this->bishops_and_queens()) |
     (this->knight_attacks(to) & this->knights()) |
     (this->king_attacks(to) & this->kings()) |
-    (this->white_pawn_attacks(to) & this->pawns_of_color(BLACK)) |
-    (this->black_pawn_attacks(to) & this->pawns_of_color(WHITE));
+    (this->white_pawn_attacks(to) & this->pawns(BLACK)) |
+    (this->black_pawn_attacks(to) & this->pawns(WHITE));
   attackers &= occ;
 
   // If the opponent has no attackers, we are finished:
@@ -1711,7 +1772,7 @@ void Position::put_piece(Piece p, Square s) {
 /// Used when setting castling rights during parsing of FEN strings.
 
 void Position::allow_oo(Color c) {
-  castleRights = CastleRights(int(castleRights) | (1+int(c)));
+  castleRights |= (1 + int(c));
 }
 
 
@@ -1719,7 +1780,7 @@ void Position::allow_oo(Color c) {
 /// Used when setting castling rights during parsing of FEN strings.
 
 void Position::allow_ooo(Color c) {
-  castleRights = CastleRights(int(castleRights) | (4+4*int(c)));
+  castleRights |= (4 + 4*int(c));
 }
 
 
@@ -1757,7 +1818,7 @@ Key Position::compute_pawn_key() const {
   Square s;
 
   for(Color c = WHITE; c <= BLACK; c++) {
-    b = this->pawns_of_color(c);
+    b = this->pawns(c);
     while(b) {
       s = pop_1st_bit(&b);
       result ^= zobrist[c][PAWN][s];
@@ -1890,6 +1951,47 @@ bool Position::is_draw() const {
 }
 
 
+/// Position::has_mate_threat() tests whether a given color has a mate in one
+/// from the current position.  This function is quite slow, but it doesn't
+/// matter, because it is currently only called from PV nodes, which are rare.
+
+bool Position::has_mate_threat(Color c) {
+  UndoInfo u1, u2;
+  Color stm = this->side_to_move();
+
+  // The following lines are useless and silly, but prevents gcc from
+  // emitting a stupid warning stating that u1.lastMove and u1.epSquare might
+  // be used uninitialized.
+  u1.lastMove = lastMove;
+  u1.epSquare = epSquare;
+
+  if(this->is_check())
+    return false;
+  
+  // If the input color is not equal to the side to move, do a null move
+  if(c != stm) this->do_null_move(u1);
+
+  MoveStack mlist[120];
+  int count;
+  bool result = false;
+
+  // Generate legal moves
+  count = generate_legal_moves(*this, mlist);
+
+  // Loop through the moves, and see if one of them is mate.
+  for(int i = 0; i < count; i++) {
+    this->do_move(mlist[i].move, u2);
+    if(this->is_mate()) result = true;
+    this->undo_move(mlist[i].move, u2);
+  }
+
+  // Undo null move, if necessary
+  if(c != stm) this->undo_null_move(u1);
+
+  return result;
+}
+
+
 /// Position::init_zobrist() is a static member function which initializes the
 /// various arrays used to compute hash keys.
 
@@ -1918,11 +2020,22 @@ void Position::init_zobrist() {
 }
 
 
-/// Position::init_piece_square_tables() initializes the black half of the
-/// midgame and endgame piece square tables by mirroring and changing the
-/// side of the corresponding white scores.
+/// Position::init_piece_square_tables() initializes the piece square tables.
+/// This is a two-step operation:  First, the white halves of the tables are
+/// copied from the MgPST[][] and EgPST[][] arrays, with a small random number
+/// added to each entry if the "Randomness" UCI parameter is non-zero.
+/// Second, the black halves of the tables are initialized by mirroring
+/// and changing the sign of the corresponding white scores.
 
 void Position::init_piece_square_tables() {
+  int r = get_option_value_int("Randomness"), i;
+  for(Square s = SQ_A1; s <= SQ_H8; s++) {
+    for(Piece p = WP; p <= WK; p++) {
+      i = (r == 0)? 0 : (genrand_int32() % (r*2) - r);
+      MgPieceSquareTable[p][s] = Value(MgPST[p][s] + i);
+      EgPieceSquareTable[p][s] = Value(EgPST[p][s] + i);
+    }
+  }
   for(Square s = SQ_A1; s <= SQ_H8; s++)
     for(Piece p = BP; p <= BK; p++) {
       MgPieceSquareTable[p][s] = -MgPieceSquareTable[p-8][flip_square(s)];
@@ -1931,6 +2044,66 @@ void Position::init_piece_square_tables() {
 }
 
 
+/// Position::flipped_copy() makes a copy of the input position, but with
+/// the white and black sides reversed.  This is only useful for debugging,
+/// especially for finding evaluation symmetry bugs.
+
+void Position::flipped_copy(const Position &pos) {
+  assert(pos.is_ok());
+
+  this->clear();
+
+  // Board
+  for(Square s = SQ_A1; s <= SQ_H8; s++)
+    if(!pos.square_is_empty(s))
+      this->put_piece(Piece(int(pos.piece_on(s)) ^ 8), flip_square(s));
+
+  // Side to move
+  sideToMove = opposite_color(pos.side_to_move());
+
+  // Castling rights
+  if(pos.can_castle_kingside(WHITE)) this->allow_oo(BLACK);
+  if(pos.can_castle_queenside(WHITE)) this->allow_ooo(BLACK);
+  if(pos.can_castle_kingside(BLACK)) this->allow_oo(WHITE);
+  if(pos.can_castle_queenside(BLACK)) this->allow_ooo(WHITE);
+
+  initialKFile = pos.initialKFile;
+  initialKRFile = pos.initialKRFile;
+  initialQRFile = pos.initialQRFile;
+
+  for(Square sq = SQ_A1; sq <= SQ_H8; sq++)
+    castleRightsMask[sq] = ALL_CASTLES;
+  castleRightsMask[make_square(initialKFile, RANK_1)] ^= (WHITE_OO|WHITE_OOO);
+  castleRightsMask[make_square(initialKFile, RANK_8)] ^= (BLACK_OO|BLACK_OOO);
+  castleRightsMask[make_square(initialKRFile, RANK_1)] ^= WHITE_OO;
+  castleRightsMask[make_square(initialKRFile, RANK_8)] ^= BLACK_OO;
+  castleRightsMask[make_square(initialQRFile, RANK_1)] ^= WHITE_OOO;
+  castleRightsMask[make_square(initialQRFile, RANK_8)] ^= BLACK_OOO;
+
+  // En passant square
+  if(pos.epSquare != SQ_NONE)
+    epSquare = flip_square(pos.epSquare);
+
+  // Checkers
+  this->find_checkers();
+
+  // Hash keys
+  key = this->compute_key();
+  pawnKey = this->compute_pawn_key();
+  materialKey = this->compute_material_key();
+
+  // Incremental scores
+  mgValue = this->compute_mg_value();
+  egValue = this->compute_eg_value();
+
+  // Material
+  npMaterial[WHITE] = this->compute_non_pawn_material(WHITE);
+  npMaterial[BLACK] = this->compute_non_pawn_material(BLACK);
+
+  assert(this->is_ok());
+}
+  
+
 /// Position::is_ok() performs some consitency checks for the position object.
 /// This is meant to be helpful when debugging.
 
