@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,14 +25,12 @@
 ////
 
 #include "bitboard.h"
-#include "castle.h"
 #include "color.h"
 #include "direction.h"
 #include "move.h"
 #include "piece.h"
 #include "phase.h"
 #include "square.h"
-#include "undoinfo.h"
 #include "value.h"
 
 
@@ -54,6 +52,34 @@ const int MaxGameLength = 220;
 //// Types
 ////
 
+/// Castle rights, encoded as bit fields:
+
+enum CastleRights {
+  NO_CASTLES = 0, 
+  WHITE_OO = 1, BLACK_OO = 2, 
+  WHITE_OOO = 4, BLACK_OOO = 8,
+  ALL_CASTLES = 15
+};
+
+
+/// The UndoInfo struct stores information we need to restore a Position
+/// object to its previous state when we retract a move.  Whenever a move
+/// is made on the board (by calling Position::do_move), an UndoInfo object
+/// must be passed as a parameter.  When the move is unmade (by calling
+/// Position::undo_move), the same UndoInfo object must be passed again.
+
+struct UndoInfo {
+  int castleRights;
+  Square epSquare;
+  Bitboard checkersBB;
+  Key key, pawnKey, materialKey;
+  int rule50;
+  Move lastMove;
+  PieceType capture;
+  Value mgValue, egValue;
+};
+
+
 /// The position data structure.  A position consists of the following data:
 ///    
 ///    * For each piece type, a bitboard representing the squares occupied
@@ -84,15 +110,17 @@ class Position {
 public:
   // Constructors
   Position();
-  Position(const Position& pos);
-  Position(const std::string& fen);
+  Position(const Position &pos);
+  Position(const std::string &fen);
 
   // Text input/output
-  void from_fen(const std::string& fen);
+  void from_fen(const std::string &fen);
   const std::string to_fen() const;
+  void print() const;
 
   // Copying
-  void copy(const Position& pos);
+  void copy(const Position &pos);
+  void flipped_copy(const Position &pos);
 
   // The piece on a given square
   Piece piece_on(Square s) const;
@@ -121,14 +149,14 @@ public:
   Bitboard rooks_and_queens() const;
   Bitboard bishops_and_queens() const;
   Bitboard sliders() const;
-  Bitboard pawns_of_color(Color c) const;
-  Bitboard knights_of_color(Color c) const;
-  Bitboard bishops_of_color(Color c) const;
-  Bitboard rooks_of_color(Color c) const;
-  Bitboard queens_of_color(Color c) const;
-  Bitboard kings_of_color(Color c) const;
-  Bitboard rooks_and_queens_of_color(Color c) const;
-  Bitboard bishops_and_queens_of_color(Color c) const;
+  Bitboard pawns(Color c) const;
+  Bitboard knights(Color c) const;
+  Bitboard bishops(Color c) const;
+  Bitboard rooks(Color c) const;
+  Bitboard queens(Color c) const;
+  Bitboard kings(Color c) const;
+  Bitboard rooks_and_queens(Color c) const;
+  Bitboard bishops_and_queens(Color c) const;
   Bitboard sliders_of_color(Color c) const;
 
   // Number of pieces of each color and type
@@ -202,6 +230,7 @@ public:
   bool move_is_pawn_push_to_7th(Move m) const;
   bool move_is_passed_pawn_push(Move m) const;
   bool move_was_passed_pawn_push(Move m) const;
+  bool move_attacks_square(Move m, Square s) const;
 
   // Information about pawns
   bool pawn_is_passed(Color c, Square s) const;
@@ -216,13 +245,13 @@ public:
   bool square_is_weak(Square s, Color c) const;
 
   // Doing and undoing moves
-  void backup(UndoInfo& u) const;
-  void restore(const UndoInfo& u);
-  void do_move(Move m, UndoInfo& u);
-  void do_move(Move m, UndoInfo& u, Bitboard dcCandidates);
-  void undo_move(Move m, const UndoInfo& u);
-  void do_null_move(UndoInfo& u);
-  void undo_null_move(const UndoInfo& u);
+  void backup(UndoInfo &u) const;
+  void restore(const UndoInfo &u);
+  void do_move(Move m, UndoInfo &u);
+  void do_move(Move m, UndoInfo &u, Bitboard dcCandidates);
+  void undo_move(Move m, const UndoInfo &u);
+  void do_null_move(UndoInfo &u);
+  void undo_null_move(const UndoInfo &u);
 
   // Static exchange evaluation
   int see(Square from, Square to) const;
@@ -243,6 +272,9 @@ public:
   bool is_mate();
   bool is_draw() const;
 
+  // Check if one side threatens a mate in one
+  bool has_mate_threat(Color c);
+
   // Number of plies since the last non-reversible move
   int rule_50_counter() const;
 
@@ -269,10 +301,10 @@ private:
 
   // Helper functions for doing and undoing moves
   void do_castle_move(Move m);
-  void do_promotion_move(Move m, UndoInfo& u);
+  void do_promotion_move(Move m, UndoInfo &u);
   void do_ep_move(Move m);
   void undo_castle_move(Move m);
-  void undo_promotion_move(Move m, const UndoInfo& u);
+  void undo_promotion_move(Move m, const UndoInfo &u);
   void undo_ep_move(Move m);
   void find_checkers();
 
@@ -304,7 +336,7 @@ private:
 
   // Other info
   Color sideToMove;
-  CastleRights castleRights;
+  int castleRights;
   File initialKFile, initialKRFile, initialQRFile;
   Square epSquare;
   Square kingSquare[2];
@@ -315,14 +347,14 @@ private:
   Value npMaterial[2];
 
   // Static variables
-  static CastleRights castleRightsMask[64];
+  static int castleRightsMask[64];
   static Key zobrist[2][8][64];
   static Key zobEp[64];
   static Key zobCastle[16];
   static Key zobMaterial[2][8][16];
   static Key zobSideToMove;
-  static int MgPieceSquareTable[16][64];
-  static int EgPieceSquareTable[16][64];
+  static Value MgPieceSquareTable[16][64];
+  static Value EgPieceSquareTable[16][64];
 };
 
 
@@ -419,35 +451,35 @@ inline Bitboard Position::sliders() const {
   return this->bishops() | this->queens() | this->rooks();
 }
 
-inline Bitboard Position::pawns_of_color(Color c) const {
+inline Bitboard Position::pawns(Color c) const {
   return this->pieces_of_color_and_type(c, PAWN);
 }
 
-inline Bitboard Position::knights_of_color(Color c) const {
+inline Bitboard Position::knights(Color c) const {
   return this->pieces_of_color_and_type(c, KNIGHT);
 }
 
-inline Bitboard Position::bishops_of_color(Color c) const {
+inline Bitboard Position::bishops(Color c) const {
   return this->pieces_of_color_and_type(c, BISHOP);
 }
 
-inline Bitboard Position::rooks_of_color(Color c) const {
+inline Bitboard Position::rooks(Color c) const {
   return this->pieces_of_color_and_type(c, ROOK);
 }
 
-inline Bitboard Position::queens_of_color(Color c) const {
+inline Bitboard Position::queens(Color c) const {
   return this->pieces_of_color_and_type(c, QUEEN);
 }
 
-inline Bitboard Position::kings_of_color(Color c) const {
+inline Bitboard Position::kings(Color c) const {
   return this->pieces_of_color_and_type(c, KING);
 }
 
-inline Bitboard Position::rooks_and_queens_of_color(Color c) const {
+inline Bitboard Position::rooks_and_queens(Color c) const {
   return this->rooks_and_queens() & this->pieces_of_color(c);
 }
 
-inline Bitboard Position::bishops_and_queens_of_color(Color c) const {
+inline Bitboard Position::bishops_and_queens(Color c) const {
   return this->bishops_and_queens() & this->pieces_of_color(c);
 }
 
@@ -512,11 +544,11 @@ inline Square Position::king_square(Color c) const {
 }
 
 inline bool Position::can_castle_kingside(Color side) const {
-  return int(castleRights) & (1+int(side));
+  return castleRights & (1+int(side));
 }
 
 inline bool Position::can_castle_queenside(Color side) const {
-  return int(castleRights) & (4+4*int(side));
+  return castleRights & (4+4*int(side));
 }
 
 inline bool Position::can_castle(Color side) const {
@@ -600,15 +632,15 @@ inline bool Position::king_attacks_square(Square f, Square t) const {
 }
 
 inline bool Position::pawn_is_passed(Color c, Square s) const {
-  return !(this->pawns_of_color(opposite_color(c)) & passed_pawn_mask(c, s));
+  return !(this->pawns(opposite_color(c)) & passed_pawn_mask(c, s));
 }
 
 inline bool Position::pawn_is_isolated(Color c, Square s) const {
-  return !(this->pawns_of_color(c) & NeighboringFilesBB[square_file(s)]);
+  return !(this->pawns(c) & neighboring_files_bb(s));
 }
 
 inline bool Position::pawn_is_doubled(Color c, Square s) const {
-  return this->pawns_of_color(c) & squares_behind(c, s);
+  return this->pawns(c) & squares_behind(c, s);
 }
 
 inline bool Position::file_is_open(File f) const {
@@ -616,11 +648,11 @@ inline bool Position::file_is_open(File f) const {
 }
 
 inline bool Position::file_is_half_open(Color c, File f) const {
-  return !(this->pawns_of_color(c) & file_bb(f));
+  return !(this->pawns(c) & file_bb(f));
 }
 
 inline bool Position::square_is_weak(Square s, Color c) const {
-  return !(this->pawns_of_color(c) & outpost_mask(opposite_color(c), s));
+  return !(this->pawns(c) & outpost_mask(opposite_color(c), s));
 }
                                 
 inline Key Position::get_key() const {
@@ -636,11 +668,11 @@ inline Key Position::get_material_key() const {
 }
 
 inline Value Position::mg_pst(Color c, PieceType pt, Square s) const {
-  return Value(MgPieceSquareTable[piece_of_color_and_type(c, pt)][s]);
+  return MgPieceSquareTable[piece_of_color_and_type(c, pt)][s];
 }
 
 inline Value Position::eg_pst(Color c, PieceType pt, Square s) const {
-  return Value(EgPieceSquareTable[piece_of_color_and_type(c, pt)][s]);
+  return EgPieceSquareTable[piece_of_color_and_type(c, pt)][s];
 }
 
 inline Value Position::mg_value() const {
@@ -707,7 +739,7 @@ inline bool Position::opposite_colored_bishops() const {
 }
 
 inline bool Position::has_pawn_on_7th(Color c) const {
-  return this->pawns_of_color(c) & relative_rank_bb(c, RANK_7);
+  return this->pawns(c) & relative_rank_bb(c, RANK_7);
 }
                         
 
