@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -35,11 +35,12 @@
 //// Prototypes
 ////
 
-extern const std::string move_to_san(Position& pos, Move m);
-extern const std::string line_to_san(const Position& pos, Move line[],
+extern const std::string move_to_san(Position &pos, Move m);
+extern Move move_from_san(Position &pos, const std::string &str);
+extern const std::string line_to_san(const Position &pos, Move line[],
                                      int startColumn, bool breakLines);
-extern const std::string pretty_pv(const Position& pos, int time, int depth,
-                                   uint64 nodes, Value score, Move pv[]);
+extern const std::string pretty_pv(const Position &pos, int time, int depth,
+                                   uint64_t nodes, Value score, Move pv[]);
 
 
 #endif // !defined(SAN_H_INCLUDED)
