@@ -1,6 +1,6 @@
 /*
   Glaurung, a UCI chess playing engine.
-  Copyright (C) 2004-2007 Tord Romstad
+  Copyright (C) 2004-2008 Tord Romstad
 
   Glaurung is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 //// Includes
 ////
 
-#include <iostream>
+#include <string>
 
 
 ////
@@ -40,13 +40,13 @@ extern bool Chess960;
 
 extern void init_uci_options();
 extern void print_uci_options();
-extern bool get_option_value_bool(const std::string& optionName);
-extern int get_option_value_int(const std::string& optionName);
-extern const std::string get_option_value_string(const std::string& optionName);
-extern bool button_was_pressed(const std::string& buttonName);
-extern void set_option_value(const std::string& optionName, 
-                             const std::string& newValue);
-extern void push_button(const std::string& buttonName);
+extern bool get_option_value_bool(const std::string &optionName);
+extern int get_option_value_int(const std::string &optionName);
+extern const std::string get_option_value_string(const std::string &optionName);
+extern bool button_was_pressed(const std::string &buttonName);
+extern void set_option_value(const std::string &optionName, 
+                             const std::string &newValue);
+extern void push_button(const std::string &buttonName);
 
 
 #endif // !defined(UCIOPTION_H_INCLUDED)
